__doc__ = """ 

dq2_get_files: script get GRID results using dq2-get command 

Usage:
chmod u+x tt19RR_sredjen/python/dq2_get_files.py
python tt19RR_sredjen/python/dq2_get_files.py --hdir=/... --indir=/... --outdir=/... --outname=file_name
"""
__author__ = "Ivan Bozic"

# Importing outside components

import os,sys,subprocess,datetime,pydoc

# Input parameters

def parseCmdLine(args):
    """ Parse input command line to optdict.
    To get the whole list of options type : run_.py -h"""
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("--dataset",dest="dataset",help="Location and name of input dataset for grid jobs", default="/TopRootCore-v00.11/dataset_data11_7TeV_p722.txt")
    parser.add_option("--prefix", dest="prefix", help="Prefix appended to the output DS name, should be something like user.yourname",default="user.%s." % os.environ['USER'])
    parser.add_option("--hdir", dest="hdir", help="Home directory",default="/afs/cern.ch/user/i/ibozic")
    parser.add_option("--outdir", dest="outdir", help="Directory containing output files",default="/eos/atlas/user/i/ibozic/tt19RR_sredjen_ver_2.2") 
    parser.add_option("--outname", dest="outname", help="Output file name",default="MergeResults.root")
    
    (config, args) = parser.parse_args(args)
    return config

def list_filenames(config):

    # Defining parameters
    
    lfolders = [config.indir]
    ldskey=[]
    dirs=[]
    filenames=[]
 
    # Prepare output files obtained on GRID analysis   

    if config.isgrid:

    # Loading data set
        
        if config.hdir in config.dataset:
            datasetname = config.dataset
        else:
            datasetname = config.hdir + config.dataset 
        infile=open(datasetname,'r+')

    # Prepare output dataset names

        for line in infile:
            line = line.rstrip(os.linesep)
            if line.endswith('/'):
                line = prefix+line
                ldskey.append(line)
            else:
                while not line.endswith('/'):
                    line=line[:-1]
                line = prefix+line
                ldskey.append(line)

    return ldskey

def main():

    config = parseCmdLine(sys.argv[1:])

    # Check does output folder exist, if not create it

    if config.hdir in config.outdir:
        outputdir = config.outdir
    else:
        outputdir = config.hdir+config.outdir
    if outputdir.endswith('/'): outputdir = outputdir[:-1]
    mynewdir = os.path.basename(outputdir)
    if not os.path.isdir(mynewdir): os.mkdir(outputdir)

    # Seting output path

    cmd = 'cd %s' % (outputdir)
    ret = subprocess.call(cmd, shell=True)

    # Getting the data

    mfilenames = list_filenames(config)
    for files in mfilenames:
        cmd = 'dq2-get %s' % files
        ret = subprocess.call(cmd, shell=True)

    return

if __name__ == "__main__":
    main()


    

    

    
