#!/usr/bin/env python

__doc__ = r"""
This script is used to get the list of dataset to consider in 0-lepton search for a given tag

Usage:
chmod u+x tt19RR_sredjen/python/get_list_dataset.py
tt19RR_sredjen/python/get_list_dataset.py --help
tt19RR_sredjen/python/get_list_dataset.py
"""

import os,sys,subprocess,datetime
from pyAMI.pyAMI import AMI
from pyAMI.amiListDatasets import amiListDatasets

# List of keywords used to remove non-used datasets
filters = ['_JetTauEtmiss']
# List of keywords used to require datasets in --minimal mode
reqfilters = ['_JetTauEtmiss'
              ]

def parseCmdLine(args):
    """ Parse input command line to optdict.
    To get the whole list of options type : get_list_dataset.py -h"""
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("--prefix", dest="prefix", help="Prefix to search datasets in AMI",default="data11_7TeV") 
    parser.add_option("--suffix", dest="suffix", help="Suffix appended to the output file name",default="") 
    parser.add_option("--tag", dest="tag", help="Production tag",default="p722") 
    parser.add_option("--minimal", dest="minimal", help="For MC select only minimal list of samples", action='store_true', default=False)
    parser.add_option("--replica", dest="replica", help="Check if CERN is among replicas", action='store_true', default=False)
    parser.add_option("--nodq2", dest="nodq2", help="Only use pyami", action='store_true', default=True)
    parser.add_option("--nocheck", dest="nocheck", help="Do not check if dataset exists ?", action='store_true', default=False)
    parser.add_option("--debug", dest="debug", help="Debug mode", action='store_true', default=False)
    (config, args) = parser.parse_args(args)
    return config

def main():
    config = parseCmdLine(sys.argv[1:])
    ami = AMI()
    loutDS = []
    # Get dataset name
    if config.nodq2: # use pyami to get the list of datasets
        dskey = config.prefix+'%.merge.NTUP_TOPJET%'+config.tag
        lcmd = [dskey,"0,10000"] # second list element defines number of output entries
        #dskey = 'mc11_7TeV.118962.AlpgenJimmyZnunubbFullNp0_pt20.merge.NTUP_SUSY.e835_s1310_s1300_r3043_r2993_p832'
        #lcmd = [dskey] # second list element defines number of output entries
        result = ami.execute(amiListDatasets().command(lcmd))
        myd = result.getDict()
        if not u'Element_Info' in myd:
            print 'AMI command dataset list is empty',myd
            return
        for key in sorted(myd[u'Element_Info']):
            dsname = str(myd[u'Element_Info'][key][u'logicalDatasetName'])
            cut = False
           # for filter in filters:
            #    if filter in dsname.split('.')[2]: cut = True
            if config.minimal:
                cut = True
                for filter in reqfilters:
                    if filter in dsname.split('.')[2]: cut = False
            # replace buggy dataset in r2700 reproduced with r2780
           #if '_r2700_' in dsname:
            #    dsname2 = dsname.replace('_r2700_','_r2780_')
             #   if 'F_pythia_jetjet' in dsname2: dsname2 = dsname2.replace('_s1309_','_s1349_')
             #   if dsname2 in loutDS: cut = True
            #elif '_r2780_' in dsname:
            #    dsname2 = dsname.replace('_r2780_','_r2700_')
            #    if 'F_pythia_jetjet' in dsname2: dsname2 = dsname2.replace('_s1349_','_s1309_')
            #    if dsname2 in loutDS: loutDS.remove(dsname2) 
            if cut: continue
            loutDS.append(dsname)
    else: # use dq2-ls to get the list of datasets
        fdq2 = os.popen('dq2-ls %s*.merge.NTUP_TOPJET.*%s/' % (config.prefix,config.tag))
        ldq2 = fdq2.readlines()
        for dq2name in ldq2:
            dq2name = dq2name.split('\n')[0]
            cut = False
            if config.minimal:
                cut = True
                for filter in reqfilters:
                    if filter in dq2name.split('.')[2]: cut = False
            if cut: continue
            loutDS.append(dq2name[:-1])

    foutname = 'dataset_'+config.prefix+'_'+config.tag+config.suffix+'.txt'
    lexist = {}
    if os.path.isfile(foutname):
        fin = open(foutname,'r')
        for line in fin:
            line = line.split('\n')[0]
            if line == "": continue
            key = line.split()[0]
            if key.endswith('/'): key = key[:-1]
            lexist[key] = line
        fin.close()
        ret = subprocess.call('cp -f %s %s' % (foutname,foutname.replace('.txt','_copy.txt')),shell=True)
    fout = open(foutname,'w')
    nevtsperperiod = {}
    for dsname in sorted(loutDS):
        print dsname
        if not config.nocheck and dsname in lexist:
            fout.write(lexist[dsname]+'\n') 
            continue
        lcmd = [dsname]
        result = ami.execute(amiListDatasets().command(lcmd))
        myd = result.getDict()
        if not u'Element_Info' in myd: 
            print dsname,'not found'
            continue

        # Check if events are available
        lcmd = ['GetDatasetInfo','logicalDatasetName=%s' % dsname]
        result = ami.execute(lcmd)
        myd2 = result.getDict()
        if myd2[u'Element_Info']['row_1'][u'prodsysStatus'] == u'ALL EVENTS AVAILABLE':
            if config.replica:
                lcmd = ['GUIDQ2Info','amiAdvanced=ON','amiLang=english','logicalDatasetName=%s' % dsname,'processingStep=production','project=data11_7TeV']
                #lcmd = ['GUIDQ2Info','amiAdvanced=ON','amiLang=english','logicalDatasetName=mc11_7TeV.106484.SU4_herwigpp_susy.merge.NTUP_SUSY01LEP.e835_s1310_s1300_r2730_r2700_p774','processingStep=production','project=mc11_001']
                #lcmd = ['help']
                #lcmd = ['GUIDQ2Info','logicalDatasetName=mc11_7TeV.106484.SU4_herwigpp_susy.merge.NTUP_SUSY.e835_a131_s1353_a133_r2780_p756','amiAdvanced=ON']
                result = ami.execute(lcmd)
                myd3 = result.getDict()
            if not dsname.endswith('/'): dsname += '/'
            nevts  = myd2[u'Element_Info']['row_1'][u'totalEvents']
            nfiles = myd2[u'Element_Info']['row_1'][u'nFiles']
            period = 'MC'
            if u'period' in myd2[u'Element_Info']['row_1']: period = myd2[u'Element_Info']['row_1'][u'period']
            if not period in nevtsperperiod: nevtsperperiod[period] = 0
            nevtsperperiod[period] += int(nevts)
            print 'adding',dsname
            fout.write("%s %s %s %s\n" % (dsname,nevts,nfiles,period))
        else:
            print dsname,'not yet ready'
    fout.close()
    print 'list of dataset dumped in',foutname
    print 'events per period (if meaningful)'
    for period in sorted(nevtsperperiod):
        print period,'=',nevtsperperiod[period]
    return

if __name__ == "__main__":
    main()
