#!/usr/bin/env python

__doc__ = """
This script can be used to launch jobs on the grid
Usage:
setup RootCore and pathena 
chmod u+x tt19RR_sredjen/python/launch_jobs_on_grid.py;
tt19RR_sredjen/python/launch_jobs_on_grid.py --help
tt19RR_sredjen/python/launch_jobs_on_grid.py
"""

import os,sys,subprocess,datetime,re

def parseCmdLine(args):
    """ Parse input command line to optdict.
    To get the whole list of options type : launch_jobs_on_grid.py -h"""
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("--inDS", dest="inDS", help="Name of single input dataset",default="") 
    parser.add_option("--inDSfile", dest="inDSfile", help="Names of input datasets stored in a file",default="") 
    parser.add_option("--prefix", dest="prefix", help="Prefix appended to the output DS name, should be something like user.yourname",default="user.%s." % os.environ['USER']) 
    parser.add_option("--suffix", dest="suffix", help="Suffix appended to the output DS name in case you need to add a version for example",default="") 
    parser.add_option("--debug", dest="debug", help="Debug mode", action='store_true', default=False)
    parser.add_option("--runopts", dest="runopts", help="Command line arguments to runttbar executable", default="")
    parser.add_option("--filesperjob", dest="filesperjob", help="Number of files to process per job!", default=10)
    parser.add_option("--container", dest="container", help="If the input datasets are in a container with multiple run numbers, configuure prun to keep them distinct", action='store_true', default=False)
    parser.add_option("--prunopts", dest="prunopts", help="Command line arguments to prun", default="")
    parser.add_option("--splitjes", dest="splitjes", help="Run all JES variations independently", action='store_true', default=False) 
    # dodajemo jos nesto
    (config, args) = parser.parse_args(args)
    return config

def main():
    config = parseCmdLine(sys.argv[1:])

    ## Get list of input datasets    
    linDS = []
    if config.inDS != "": 
        linDS.append(config.inDS)
    elif config.inDSfile != "":
        finDS = open(config.inDSfile,'r')
        for inDS in finDS:
            linDS.append(inDS.strip('\n'))
    if config.debug: print 'list of datasets',linDS
    if len(linDS) == 0:
        print "You did not specify any input dataset or file containing a list of input datasets.\nPlease type launch_jobs_on_grid.py -h"
        sys.exit(0)

  ## Store SUSYTools/data root files in a tar.gz used as external file
    rootCoreDir = os.environ['ROOTCOREDIR']
    lrootfiles = []
    for filename in os.listdir(rootCoreDir+'/../tt19RR_sredjen/util/'):
        if not (filename.endswith('.root') or filename.endswith('.xml')
                or filename.endswith('.env')): continue
        lrootfiles.append(rootCoreDir+'/../tt19RR_sredjen/util/'+filename)
    cmd = 'gtar cvzf externalFiles.tar.gz'
    for rootfile in lrootfiles:
        cmd += ' '+rootfile
    if config.debug:
        print cmd
        config.filesperjob = 1
    ret = subprocess.call(cmd, shell=True)
    if ret != 0:
        print 'failed to build external tar.gz file'
        sys.exit(0)

    ## Loop over input datasets
    first = True

    if config.splitjes:
        jeslist = ['none','JESUP','JESDOWN','JER']
        config.runopts += ' -jes _JESVAR_'
        config.suffix += '.jes_JESVAR_'

    for inDS in linDS:
        if inDS.startswith("#"): continue
        print 'launching grid job for',inDS
        outDS = re.sub('^(user\.[a-zA-Z]*\.)?',config.prefix,inDS)
        if config.suffix != "":
            if outDS.endswith('/'):
                outDS = outDS.split('/')[0]+config.suffix+'/'
            else:
                outDS += config.suffix
        if len(outDS) > 130: # panda will complain dataset name is too long
            if '.merge' in outDS: outDS = outDS.replace('.merge',"")
            if '.NTUP_TOPJET' in outDS: outDS = outDS.replace('.NTUP_TOPJET',"")
        
        print 'output DS: ' + outDS
        scriptcmd = r"""gtar -xvzf externalFiles.tar.gz ; \$ROOTCOREDIR/../tt19RR_sredjen/bin/runttbar -in \%IN -out output.root"""
	#scriptcmd = r"""\$ROOTCOREDIR/../tt19RR_sredjen/bin/runttbar -in \%IN -out rez.root"""       
	if "data11" in inDS: scriptcmd += " -data 1 -grl 1"
        scriptcmd += " " + config.runopts
        cmd = r"""prun --exec "%(scriptcmd)s" --useRootCore --athenaTag AtlasProduction,17.3.1.1 --inDS %(inDS)s --outputs rez.root --outDS %(outDS)s --nFilesPerJob %(filesperjob)s %(prunopts)s""" % {'inDS':inDS,'outDS':outDS,'scriptcmd':scriptcmd,'filesperjob':config.filesperjob,'prunopts':config.prunopts}
        if config.container and not 'data11' in inDS:
            cmd += ' --useContElementBoundary'
        if 'data11' in inDS and not config.debug:
            cmd = cmd.replace(
                '--nFilesPerJob {0}'.format(config.filesperjob),
                '--nGBPerJob=MAX')
        if config.debug:
            cmd += ' --nFiles 1'
            print cmd
        if len(linDS) > 1 or config.splitjes:
            if first and not 'inTarBall' in cmd:
                first = False
                cmd += ' --outTarBall=/tmp/'+os.environ['USER']+'/TarBall_prun.tar.gz'
            else: 
                cmd += ' --inTarBall=/tmp/'+os.environ['USER']+'/TarBall_prun.tar.gz'

        if config.splitjes:
            for jesvar in jeslist:
                cmdcopy = cmd.replace('_JESVAR_',jesvar)
                if not jesvar == 'none':
                    cmdcopy = cmdcopy.replace('outTarBall','inTarBall')

                ret = subprocess.call(cmdcopy, shell=True)
                if ret != 0:
                    print 'command failed => stop script'
                    sys.exit(0)
        else:
            ret = subprocess.call(cmd, shell=True)
            if ret != 0:
                print 'command failed => stop script'
                sys.exit(0)

    return

if __name__ == "__main__":
    main()
