__doc__ = """ 

merge_root_files: script intended to merge all *.root files which are obtained as a results of particles job 

Usage:
chmod u+x tt19RR_sredjen/python/merge_root_files.py
python tt19RR_sredjen/python/merge_root_files.py --hdir=/... --indir=/... --outdir=/... --outname=file_name
"""
__author__ = "Ivan Bozic"

# Importing outside components

import os,sys,subprocess,datetime,pydoc

from ROOT import *

# Input parameters

def parseCmdLine(args):
    """ Parse input command line to optdict.
    To get the whole list of options type : run_.py -h"""
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("--isgrid", dest="isgrid", help="Where is job started (defauilt is LBS)", default=True)
    parser.add_option("--dataset",dest="dataset",help="Location and name of input dataset for grid jobs", default="/TopRootCore-v00.11/dataset_data11_7TeV_p722.txt")
    parser.add_option("--hdir", dest="hdir", help="Home directory",default="/afs/cern.ch/user/i/ibozic")
    parser.add_option("--indir", dest="indir", help="Directory containing input files",default="/eos/atlas/user/i/ibozic/tt19RR_sredjen_ver_2.2") 
    parser.add_option("--outdir", dest="outdir", help="Directory containing output files",default="/eos/atlas/user/i/ibozic/tt19RR_sredjen_ver_2.2") 
    parser.add_option("--outname", dest="outname", help="Output file name",default="MergeResults.root")
    
    (config, args) = parser.parse_args(args)
    return config

def list_filenames(config):

    # Defining parameters
    
    lfolders = [config.indir]
    ldskey=[]
    dirs=[]
    filenames=[]
 
    # Prepare output files obtained on GRID analysis   

    if config.isgrid:

    # Loading data set
        
        if config.hdir in config.dataset:
            datasetname = config.dataset
        else:
            datasetname = config.hdir + config.dataset    
        infile=open(datasetname,'r+')
        for line in infile:
            line = line.rstrip(os.linesep)
            if line.endswith('/'): ldskey.append(line)
            else:
                while not line.endswith('/'):
                    line=line[:-1]
                ldskey.append(line)

    # Find folders which are in dataset

        for folder in lfolders:
            ffolder = os.popen('ls %s' % folder)
            for foldername in ffolder:
                foldername = foldername.rstrip(os.linesep)
                for dskey in ldskey:
                    if dskey.endswith('/'): dskey = dskey[:-1] 
                    if dskey in foldername:
                        dirname= folder + '/' + foldername
                        dirs.append(dirname)

    # Finding *.root files in folders

        nfiles = 0
        for mydir in dirs:
            ffiles = os.popen('ls %s' % mydir)
            for fname in ffiles:
                fname = fname.split('\n')[0]
                if fname == "": continue
                fname = fname.split()[len(fname.split())-1]
                shortname = os.path.basename(fname)
                if not '.root' in shortname: continue
                nfiles += 1
                filename = mydir+'/'+shortname
                filenames.append(filename)
        print nfiles

        return filenames

    # Prepare output files obtained in LSB analysis

    else:

    # Loading database

        from database_cern1 import data
        dbkeys = sorted(data.keys())

    # Finding all datasets

        for sample in ['physics_JetTauEtmiss']:
            for key in dbkeys:
                if sample in key:
                    ldskey.append(key)

    # Prepare folders

        for folder in lfolders:
            for dskey in ldskey:
                dirname = folder+'/'+dskey
                dirs.append(dirname)
                nfiles = 0

    # Finding *.root files in folders

        for mydir in dirs:
            ffiles = os.popen('xrd eosatlas ls %s' % mydir)
            for fname in ffiles:
                fname = fname.split('\n')[0]
                if fname == "": continue
                fname = fname.split()[len(fname.split())-1]
                shortname = os.path.basename(fname)
                if not '.root' in shortname: continue
                nfiles += 1
                filename = mydir+'/'+shortname
                filenames.append(filename)
        print nfiles

        return filenames

# Main program

def main():

    config = parseCmdLine(sys.argv[0:])

    # Check does output folder exist, if not create it

    if config.hdir in config.outdir:
        outputdir = config.outdir
    else:
        outputdir = config.hdir+config.outdir
    if outputdir.endswith('/'): outputdir = outputdir[:-1]
    mynewdir = os.path.basename(outputdir)
    if not os.path.isdir(mynewdir): os.mkdir(outputdir)

    # Adding *.root files in one outputfile

    out = outputdir + '/' + config.outname
    source =''
    mfilenames = list_filenames(config)
    for files in mfilenames:
        print files
        if config.isgrid:
            source += files + ' '
        else:
            source += config.hdir + files + ' '
    cmd = 'hadd -f %s %s' % (out,source)
    ret = subprocess.call(cmd, shell=True)

    return

if __name__ == "__main__":
    main()
