#!/usr/bin/env python

""" prepare_database_cern.py: script intended to prepare a list of datasets available at cern. 

Usage:
chmod u+x tt19RR_sredjen/prepare_database_cern.py
tt19RR_sredjen/prepare_database_cern.py --help
tt19RR_sredjen/prepare_database_cern.py --tag=pXXX
"""
__author__ = "Renaud Bruneliere"

# List all folder which should be checked
# Be careful the order is important. If datasets are found with same name, first one will be picked
lfolders = [
"/eos/atlas/user/m/mmarjano",
"/eos/atlas/user/i/ijakonic"
]

import os,sys,subprocess

def parseCmdLine(args):
    """ Parse input command line to optdict.
    To get the whole list of options type : prepare_database_cern.py -h"""
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("--tag", dest="tag", help="Production tag",default="_p722") 
    (config, args) = parser.parse_args(args)
    return config

def finddatasetname(fname,config):
    isfound = False
    dsname = os.path.basename(fname)
    for startstr in ['data11_7TeV']:
        if startstr in dsname: 
            dsname = startstr+dsname.split(startstr)[1]
            isfound = True
    if not isfound: return None
    if not config.tag in dsname: return None
    if not ('.NTUP_TOPJET.' in dsname): return None
    dsname = dsname.split(config.tag)[0]+config.tag
    return dsname

def getlistdatasets(config):
    ldatasets = {}
    for folder in lfolders:
        cmd = 'xrd eosatlas ls %s' % folder
        ffiles = os.popen(cmd)
        lfiles = ffiles.readlines()
        for fname in lfiles:
            fname = fname.split('\n')[0]
            if fname == "": continue
            fname = fname.split()[len(fname.split())-1]
            dsname = finddatasetname(fname,config)
            if dsname == None: continue
            print dsname
            key = dsname.split('.')[0]+'.'+dsname.split('.')[1]+'.'+dsname.split('.')[2]
            if key in ldatasets:
                if not ldatasets[key]['dir'][0].startswith(folder.split()[0]): continue
                ldatasets[key]['dir'].append(fname)
            else:
                ldataset = {}
                ldataset['dsname'] = dsname
                ldataset['dir'] = []
                ldataset['dir'].append(fname)
                ldatasets[key] = ldataset
    return ldatasets

def main():
    config = parseCmdLine(sys.argv[1:])
    ldatasets = getlistdatasets(config)
    dirname = sys.argv[0].split(os.path.basename(sys.argv[0]))[0]
    fdbname = 'database_cern.py'
    if dirname != "": fdbname = dirname+fdbname
    fdatabase = open(fdbname,'w')
    fdatabase.write('data = {\n')
    for dataset in sorted(ldatasets):
        fdatabase.write('"%s":%s,\n' % (dataset,ldatasets[dataset]))
    fdatabase.write('}\n')
    fdatabase.close()
if __name__ == "__main__":
    main()
