#!/usr/bin/env python
__doc__    = """
Python script used to run executables in particular at cern with access to eos.
Usage: 
chmod u+x tt19RR_sredjen/python/run_.py
./tt19RR_sredjen/python/run_.py --help
./tt19RR_sredjen/python/run_.py
"""

import os,sys,subprocess,datetime

def parseCmdLine(args):
    """ Parse input command line to optdict.
    To get the whole list of options type : run_.py -h"""
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("--dskey", dest="dskey", help="Directory containing input files",
                      default="data11_7TeV.00186182.physics_JetTauEtmiss") 
    parser.add_option("--indir", dest="indir", help="Directory containing input files",default="") 
    parser.add_option("--outname", dest="outname", help="Output file name",default="rez.root") 
    parser.add_option("--stream", dest="stream", help="Stream name (if any)",default="") 
    parser.add_option("--jes", dest="jes", help="Systematics mode: JESUP,JESDOWN,JER,all,none",default="none") 
    parser.add_option("--isdata", dest="isdata", help="Data or Monte Carlo ?",
                      action='store_true', default=False) 
    parser.add_option("--isd4pd", dest="isd4pd", help="Are we running on D4PDs ?",
                      action='store_true', default=False) 
    parser.add_option("--puwgt", dest="puwgt", help="Apply pileup reweighting to MC ?",
                      action='store_true', default=False)
    parser.add_option("--sf", dest="sf", help="Apply scale factors to MC ?",
                      action='store_true', default=False)
    parser.add_option("--grl", dest="grl", help="Check GoodRunsList ?",
                      action='store_true', default=False)
    parser.add_option("--hfor", dest="hfor", help="Reject overlap between light and HF events on alpgen samples",
                      action='store_true', default=False)
    parser.add_option("--truthfilt", dest="truthfilt", help="Reject events not passing the truth filter",
                      action='store_true', default=False)
    parser.add_option("--batchmode", dest="batchmode", help="Batch mode ?",
                      action='store_true', default=False)
    parser.add_option("--allhist", dest="allhist", help="Keep only subset of histos (False) or all (True) ?",
                      action='store_true', default=False)
    parser.add_option("--ifirst", dest="ifirst", type="int", help="Copy only file number >= ifirst",default=0) 
    parser.add_option("--ilast", dest="ilast", type="int", help="Copy only file number <= ilast",default=10000000) 
    (config, args) = parser.parse_args(args)
    return config

def getlistinfiles(indir,config):
    ifile = 0
    infiles = ""
    for mydir in indir:
        if ifile > config.ilast: break
        if mydir.endswith('/'): mydir = mydir[:-1]
        if mydir.startswith('/eos/'): 
            mynewdir = os.path.basename(mydir)
            if not config.batchmode: mynewdir = '/tmp/'+os.environ['USER']+'/'+mynewdir
            if not os.path.isdir(mynewdir): os.mkdir(mynewdir)
            ffiles = os.popen('xrd eosatlas ls %s' % mydir)
        else:
            mynewdir = mydir
            ffiles = os.popen('ls %s' % mydir)
        lfiles = ffiles.readlines()
        for fname in sorted(lfiles):
            fname = fname.split('\n')[0]
            if fname == "": continue
            if (len(fname.split()) > 1): fname = fname.split()[len(fname.split())-1]
            if not '.root' in fname: continue
            if ifile > config.ilast: break
            if ifile >= config.ifirst and ifile <= config.ilast:
                if mydir.startswith('/eos/'):
                    if os.path.exists(mynewdir+os.path.basename(fname)):
                        print 'copyeosfiles:',mynewdir+os.path.basename(fname),'already copied => skip'
                    else:
                        cmd = 'xrdcp root://eosatlas/%s %s' % (fname,mynewdir)
                        ret = subprocess.call(cmd,shell=True)
                        if ret != 0: print 'copyeosfiles: problem while copying',fname
                if infiles != "": infiles += ","
                infiles += mynewdir+'/'+os.path.basename(fname)
            ifile += 1
    return infiles

def main():
    print "run_.py: starting",datetime.datetime.now()
    config = parseCmdLine(sys.argv[1:])
    bindir = os.environ['ROOTCOREDIR']+'/bin'
    if not bindir in os.environ['PATH']: os.environ['PATH'] = bindir + ":" + os.environ['PATH']
    pydir = os.environ['ROOTCOREDIR']+'/python/tt19RR_sredjen'
    if not pydir in os.environ['PYTHONPATH']: os.environ['PYTHONPATH'] = pydir + ":" + os.environ['PYTHONPATH']

    # Database containing link to all datasets stored on eos at CERN
    # To produce such a database please run
    # tt19RR_sredjen/python/prepare_database_cern.py --tag=pXXX
    from database_cern import data

    indir = []
    if config.indir == "":
        if config.dskey in data:
            [ indir.append(mydir) for mydir in data[config.dskey]['dir'] ]
    else:
        indir.append(config.indir)

    if len(indir) == 0:
        print "No input folder provided => stop"
        sys.exit(0)

    # Get list on input files
    infiles = getlistinfiles(indir,config)

    cmd = 'runttbar -in %s ' % (infiles)

    # Run executable
    #cmd = 'runttbar -in %s -out %s -data %d -d4pd %d -pu %d -sf %d -grl %d -allhist %d -jes %s -hfor %d -truthfilt %d' % (infiles,os.path.basename(config.outname),config.isdata,config.isd4pd,config.puwgt,config.sf,config.grl,config.allhist,config.jes,config.hfor,config.truthfilt)

    cmd += ' >& %s.log' % os.path.basename(config.outname).split('.root')[0]
    print 'Command launched'
    print cmd
    ret = subprocess.call(cmd, shell=True)
    ret = subprocess.call('gzip %s.log' % os.path.basename(config.outname).split('.root')[0], shell=True)

    # Copy output file
    outdir = os.path.dirname(config.outname)
   
    if outdir != "":
        if outdir.startswith("/eos/"):
            cmd = 'xrd eosatlas rm %s' % config.outname
            ret = subprocess.call(cmd, shell=True)
            cmd = 'xrd eosatlas rm %s.log.gz' % config.outname.split('.root')[0]
            ret = subprocess.call(cmd, shell=True)
            cmd = 'xrdcp %s root://eosatlas/%s' % ('rez.root',config.outname)
            cmd2 = 'xrdcp %s.log.gz root://eosatlas/%s.log.gz' % (os.path.basename(config.outname).split('.root')[0],config.outname.split('.root')[0])
        else:
            cmd = 'cp -f %s %s' % (os.path.basename(config.outname),config.outname)
            cmd2 = 'cp -f %s.log.gz %s.log.gz' % (os.path.basename(config.outname).split('.root')[0],config.outname.split('.root')[0])
        ret = subprocess.call(cmd, shell=True)
        ret = subprocess.call(cmd2, shell=True)
   
    print "run_.py: ending",datetime.datetime.now()
    return

if __name__ == "__main__":
    main()
