#!/usr/bin/env python

__doc__ = """
This script can be used to launch batch jobs (at lxbatch)
Usage:
chmod u+x tt19RR_sredjen/python/submit_batch_jobs.py;
tt19RR_sredjen/python/submit_batch_jobs.py
"""

import os,sys,subprocess,datetime

# Database containing link to all datasets stored on eos at CERN
# To produce such a database please run
# chiSelector/python/prepare_database.py 

pydir = os.environ['ROOTCOREDIR']+'/python/tt19RR_sredjen'
if not pydir in os.environ['PYTHONPATH']: os.environ['PYTHONPATH'] = pydir + ":" + os.environ['PYTHONPATH']
from database_cern import data
dbkeys = sorted(data.keys())

# General settings
queue  = {'default':'1nh'} # '8nm' for test
outdir = "/eos/atlas/user/m/mmarjano/TopRootCore-v00.11" #os.getcwd() 
maxfilesperjob = {'default':5}

##
## Below is the list of dskey from database which should be sent as batch jobs
##
ldskey = []

for sample in ['data11_7TeV.00183544.physics_JetTauEtmiss']:
    for key in dbkeys:
        if sample in key:
            ldskey.append(key)

#for sample in ['T1']:
#    for key in dbkeys:
#        if sample in key:
#            ldskey.append(key)

#for sample in ['AlpgenJimmyttbar','AlpgenJimmyttcc','AlpgenJimmyttbb',
#               'PythiaMadgraph_ttbarW','PythiaMadgraph_ttbarZ',
#               'st_schan','st_tchan','Wt']:
#    for key in dbkeys:
#        if sample in key:
#            ldskey.append(key)
# VB+jets
#for sample in ['data11_7TeV.00183544.physics_JetTauEtmiss.merge.NTUP_TOPJET.r2603_p659_p694_p722_tid524592_00']:
#		'AlpgenJimmyWenuNp','AlpgenJimmyWmunuNp','AlpgenJimmyWtaunuNp'
#               'AlpgenJimmyWbbFullNp', #'AlpgenWcNp','AlpgenWccFullNp',
#               'AlpgenJimmyZnunuNp','AlpgenJimmyZnunubbFullNp',
#               'AlpgenJimmyZee','AlpgenJimmyZmumu','AlpgenJimmyZtautau',
#               'AlpgenJimmyGamNp','SherpaZ','SherpaW','SherpaY']:
#    for key in dbkeys:
#        if sample in key:
#            ldskey.append(key)
#ldskey.append(key)
# Diboson
#for sample in ['SherpaWW','SherpaWZ','SherpaZZ','SherpaWto','SherpaZto','AlpgenJimmyWgammaNp']:
#    for key in dbkeys:
#        if sample in key:
#            ldskey.append(key)
# QCD
# Currenlty missing: 
# for pthatbin in range(9):
#     for key in dbkeys:
#         if ('J%d_pythia_jetjet' % pthatbin) in key: 
#             ldskey.append(key)

def main():
    if not os.path.isdir('batchJobs'): os.mkdir("batchJobs")
    fmainscript = open('script_to_submit_batch_jobs.sh','w')
    fmainscript.write('#!/usr/bin/env bash\n')

    maindir = os.path.dirname(os.environ['ROOTCOREDIR'])

    ## Loop over input datasets
    for dskey in ldskey:
        print 'preparing batch jobs for',dskey
        # Count number of root files
        nfiles = 0
        for mydir in data[dskey]['dir']:
            ffiles = os.popen('xrd eosatlas ls %s' % mydir)
            for fname in ffiles:
                fname = fname.split('\n')[0]
                if fname == "": continue
                fname = fname.split()[len(fname.split())-1]
                shortname = os.path.basename(fname)
                if not '.root' in shortname: continue
                nfiles += 1

        # Compute number of jobs for this dataset and make one script per job
        nfilesperjob = maxfilesperjob['default']
        if dskey in maxfilesperjob: nfilesperjob = maxfilesperjob[dskey]
	njobs = nfiles/nfilesperjob        
	#if nfiles%nfilesperjob != 0 njobs = nfiles/nfilesperjob + 1  else nfiles/nfilesperjob
        for ijob in range(njobs):
            batchname = '%s/batchJobs/%s_%d.sh' % (os.getcwd(),dskey,ijob)
            if os.path.exists(batchname):
                ret = subprocess.call('rm -f %s' % batchname, shell=True)
            fbatch = open(batchname,'w')
            fbatch.write('#!/usr/bin/env bash\n')
            fbatch.write('. /afs/cern.ch/sw/lcg/external/gcc/4.3.2/x86_64-slc5/setup.sh \n')
            fbatch.write('source /afs/cern.ch/atlas/offline/external/GRID/ddm/DQ2Clients/setup.sh \n')
            fbatch.write('source /afs/cern.ch/user/m/mmarjano/scratch0/TopRootCore-v00.11/RootCore/scripts/setup.sh \n') 
            cmd = '%s/tt19RR_sredjen/python/run_.py --batchmode --dskey=%s --outname=%s --ifirst=%d --ilast=%d' % (maindir,dskey,outdir+'/'+dskey+('_%d.root' % ijob),ijob*nfilesperjob,(ijob+1)*nfilesperjob-1)
            fbatch.write(cmd+'\n')
            fbatch.close()
            fmainscript.write("chmod u+x %s\n" % batchname)
            queuename = queue['default']
            #if dskey in queue: queuename = queue[dskey]outdir+'/'+dskey+('_%d.root' % ijob),
            fmainscript.write("bsub -q %s %s\n" % (queuename,batchname))
    fmainscript.close()
    print '\n\n================================='
    print 'To send jobs please do:'
    print 'chmod u+x script_to_submit_batch_jobs.sh'
    print './script_to_submit_batch_jobs.sh'
    print '=================================\n\n'
    return

if __name__ == "__main__":
    main()
